﻿using Newtonsoft.Json.Linq;
using Struct.PIM.Api.Client;
using Struct.PIM.Api.Models.Product;
using Struct.PIM.Api.Models.Shared;
using StructPimApiDemo.Entity;
using StructPimApiDemo.DTO;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Struct.PIM.Api.Models.Variant;
using Struct.PIM.Api.Models.GlobalList;

namespace StructPimApiDemo
{
    class Program
    {
        private static StructPIMApiClient pimFacade;
        private const string _englishCultureCode = "en-GB";
        static void Main(string[] args)
        {
            pimFacade = new StructPIMApiClient(ConfigurationManager.AppSettings["Struct.PIM.Api.Url"], ConfigurationManager.AppSettings["Struct.PIM.Api.ApiKey"]);

            while (true)
            {
                Console.WriteLine("What do you want to do?");

                Console.WriteLine("1: Get products from PIM");
                Console.WriteLine("2: Get products from PIM by ids");
                Console.WriteLine("3: Get product attributevalues");
                Console.WriteLine("4: Get specific product attributevalues");
                Console.WriteLine("5: Get variants from PIM");
                Console.WriteLine("6: Get variant attributevalues");
                Console.WriteLine("7: Get specific variant attributevalues");
                Console.WriteLine("8: Get globallist values from PIM");
                Console.WriteLine("9: Get variants updated within the last year");
                Console.WriteLine("a: Get categories by catalogue");
                Console.WriteLine("b: Get products in specific catalogue");
                Console.WriteLine("c: Create product with variant");
                Console.WriteLine("d: Update product");
                Console.WriteLine("e: Update variant");
                Console.WriteLine("f: Update value on globallist");
                Console.WriteLine("g: Search for products with brand");

                Console.WriteLine("Enter the number corresponding to the action you want to execute.");


                var key = Console.ReadKey();

                Console.WriteLine();
                Console.WriteLine();

                switch (key.KeyChar)
                {
                    case '1':
                        GetProductsFromPIM();
                        break;
                    case '2':
                        GetProductsWithSpecificIds();
                        break;
                    case '3':
                        GetProductAttributeValues();
                        break;
                    case '4':
                        GetSpecificProductAttributeValues();
                        break;
                    case '5':
                        GetVariantsFromPIM();
                        break;
                    case '6':
                        GetVariantAttributeValues();
                        break;
                    case '7':
                        GetSpecificVariantAttributeValues();
                        break;
                    case '8':
                        GetGlobalListValuesFromPIM();
                        break;
                    case '9':
                        GetUpdatedVariantsLastYear();
                        break;
                    case 'a':
                        GetCategoriesByCatalgoue();
                        break;
                    case 'b':
                        GetProductsInElectronicsSiteCatalogue();
                        break;
                    case 'c':
                        CreateProductWithVariants();
                        break;
                    case 'd':
                        UpdateProduct();
                        break;
                    case 'e':
                        UpdateVariant();
                        break;
                    case 'f':
                        UpdateGlobalListValue();
                        break;
                    case 'g':                        
                        Console.WriteLine("Enter the name of the brand you wish to search for...");
                        var value = Console.ReadLine();
                        GetProductsByBrand(value);
                        break;

                    default:
                        break;
                }

                Console.WriteLine();
            }
        }

        private static void GetProductsFromPIM()
        {            
            var products = pimFacade.Products.GetProducts();
            Console.WriteLine("Products fetched from Api");
        }

        private static void GetProductsWithSpecificIds()
        {
            var productIds = new List<int> { 48, 41 };
            var products = pimFacade.Products.GetProducts(productIds);
            Console.WriteLine("Products fetched from Api");
        }

        private static void GetProductAttributeValues()
        {
            var productId = pimFacade.Products.GetProductIds().FirstOrDefault();
            var products = pimFacade.Products.GetProductAttributeValues(productId);
            Console.WriteLine("Products fetched from Api");
        }
        private static void GetSpecificProductAttributeValues()
        {
            
            var productIds = pimFacade.Products.GetProductIds().Take(100).ToList();
            var attributeAliases = new List<string> { "Country", "Year", "Grapes" };
            var products = pimFacade.Products.GetProductAttributeValues(new ProductValuesRequestModel 
            { 
                ProductIds = productIds,
                Aliases = attributeAliases,
                IncludeValues = ValueIncludeMode.Aliases
            });

            foreach (var product in products)
            {
                var value = JObject.FromObject(product.Values).ToObject<ProductDTO>();
                var country = value.Country?.Name?.FirstOrDefault(y => y?.CultureCode == _englishCultureCode)?.Data;
                Console.WriteLine("Product is from: " + (!string.IsNullOrEmpty(country) ? country : "Unknown"));
            }            
        }
        private static void GetVariantsFromPIM()
        {            
            var variants = pimFacade.Variants.GetVariants();
            Console.WriteLine("Variants fetched from Api");
        }
        private static void GetVariantAttributeValues()
        {
            var variantId = pimFacade.Variants.GetVariantIds().FirstOrDefault();
            var variant = pimFacade.Variants.GetVariantAttributeValues(variantId);
            Console.WriteLine("variant fetched from Api");
        }
        private static void GetSpecificVariantAttributeValues()
        {
            var variantIds = pimFacade.Variants.GetVariantIds();
            var attributeAliases = new List<string> { "CPU", "HDD" };
            var variants = pimFacade.Variants.GetVariantAttributeValues(new VariantValuesRequestModel
            {
                VariantIds = variantIds,
                Aliases = attributeAliases,
                IncludeValues = ValueIncludeMode.Aliases
            });

            foreach (var variant in variants)
            {
                var value = JObject.FromObject(variant.Values).ToObject<VariantDTO>();
                var manufacturer = value.CPU?.CPUManufacturer?.Name;
                if (!string.IsNullOrEmpty(manufacturer))
                {
                    Console.WriteLine("Variant is manufactured: " + manufacturer);
                }                
            }
        }
        private static void GetUpdatedVariantsLastYear()
        {
            //Get variants where the product or the variant itself has been updated withing the last year
            var filters = new List<FieldFilterModel> { new FieldFilterModel { FieldUid = "PIM_LastModified", FilterValue = "-365d", QueryOperator = QueryOperator.LargerThan } };
            var updatedVariantIds = SearchVariants(filters);


            var variants = pimFacade.Variants.GetVariants(updatedVariantIds);
            Console.WriteLine($"{updatedVariantIds.Count} updated variant ids returned");
            Console.WriteLine($"Fetched data from {variants.Count} variants");
        }

        private static void GetGlobalListValuesFromPIM()
        {
            var globalList = pimFacade.GlobalLists.GetGlobalList("Colors");
            var globalListValues = pimFacade.GlobalLists.GetGlobalListValues<Color>(globalList.Uid);
            var colors = globalListValues.GlobalListValues;
            Console.WriteLine(string.Join(", ", colors.Take(100).Select(x => x.Value.Name.FirstOrDefault(y => y.CultureCode == _englishCultureCode).Data)));
        }

        private static void GetCategoriesByCatalgoue()
        {
            var catalogueUids = pimFacade.Catalogues.GetCatalogueUids();
            var catalogueRootNodes = pimFacade.Catalogues.GetCatalogueChildren(catalogueUids.FirstOrDefault());
            Console.WriteLine($"Got {catalogueRootNodes.Count()} catalogues from PIM");
        }

        private static void GetProductsInElectronicsSiteCatalogue()
        {
            var categories = pimFacade.Catalogues.GetCatalogueDescendants(new Guid("bf13f53c-9e48-4bb1-bd0c-6ced03bc8561"));
            var productsInCategories = pimFacade.Catalogues.GetProductsInCategories(categories.Categories.Select(x => x.Id).ToList());
            Console.WriteLine("Number of products: " + productsInCategories.Values.SelectMany(x => x).Count());
        }

        private static void CreateProductWithVariants()
        {
            var createModel = new CreateProductModel<ProductDTO>();

            createModel.ProductStructureUid = new Guid("df5733d0-af60-4299-9449-1bed2f4f9ba1");
            createModel.VariationDefinitionUid = new Guid("d2110e80-0bc2-4913-a4b2-15a320dc5def");
            createModel.PrimaryCategoryId = 52;
            createModel.Values = new ProductDTO();

            createModel.Values.ModelNumber = "Q60TAU" + new Random().Next(1000);
            createModel.Values.Name = new List<LocalizedData<string>> { new LocalizedData<string> { CultureCode = _englishCultureCode, Data = "Samsung 2020 QLED TV Q60TAU" } };
            createModel.Values.Brand = new Brand { Name = "Samsung" };
            createModel.Values.PrimaryImage = "1237";

            pimFacade.Products.CreateProduct(createModel);

            var productId = pimFacade.Products.GetInternalId("ProductIdentifier", createModel.Values.ModelNumber);

            CreateVariant(productId);

        }

        private static void CreateVariant(int? productId)
        {
            var createModel = new CreateVariantModel<VariantDTO>() { ProductId = productId.Value };

            createModel.Values = new VariantDTO();
            createModel.Values.SKU =  "QE55Q60TAU" + new Random().Next(1000);
            createModel.Values.Wifi = true;
            createModel.Values.ExtraImages = new List<string> { "1238" };
            createModel.Values.ScreenSize = 55;

            pimFacade.Variants.CreateVariant(createModel);
        }

        private static void UpdateProduct()
        {
            var identifierMap = pimFacade.Products.GetInternalIdMap("ProductIdentifier");
            int? productId = identifierMap.FirstOrDefault(x => x.Key.Contains("Q60TAU")).Value;
            var updateModel = new UpdateProductModel();
            updateModel.Values = new Dictionary<string, dynamic>();
            updateModel.Values.Add("Brand", new Brand { Name = "LG" });

            if (productId.HasValue)
                pimFacade.Products.UpdateProduct(productId.Value, updateModel);
        }

        private static void UpdateVariant()
        {
            var identifierMap = pimFacade.Variants.GetInternalIdMap("VariantIdentifier");
            int? variantId = identifierMap.FirstOrDefault(x => x.Key.Contains("QE55")).Value;
            var updateModel = new UpdateVariantModel();
            updateModel.Values.Add("Wifi", false);

            if (variantId.HasValue)
                pimFacade.Variants.UpdateVariant(variantId.Value, updateModel);
        }

        private static void UpdateGlobalListValue()
        {
            var globalList = pimFacade.GlobalLists.GetGlobalList("Colors");
            var globalListValues = pimFacade.GlobalLists.GetGlobalListValues<Color>(globalList.Uid);
            var updateModel = new List<GlobalListValue<Color>>();

            foreach (var value in globalListValues.GlobalListValues)
            {
                if (string.IsNullOrEmpty(value.Value.HexColor))
                {
                    value.Value.HexColor = value.Value.BaseColor.HexColor;
                }
                updateModel.Add(new GlobalListValue<Color>
                {
                    Uid = value.Uid,
                    Value = value.Value,
                    SortOrder = value.SortOrder
                });
            }

            pimFacade.GlobalLists.UpdateGlobalListValues(globalList.Uid, updateModel);
        }

        private static void GetProductsByBrand(string value)
        {
            var fields = pimFacade.Products.GetQueryableFields();
            var brandField = fields.FirstOrDefault(x => x.Name.ToLower() == "brand");

            var filters = new List<FieldFilterModel> { new FieldFilterModel { FieldUid = brandField.Uid, FilterValue = value, QueryOperator = QueryOperator.Equals } };
            var variants = SearchProducts(filters);

            variants.ForEach(x => Console.WriteLine("Internal id: " + x));
        }

        private static List<int> SearchVariants(List<FieldFilterModel> filters)
        {
            return pimFacade.Variants.Search(new SearchModel
            {
                IncludeArchived = true,
                QueryModel = new SimpleQueryModel
                {
                    BooleanOperator = BooleanOperator.And,
                    Filters = filters                    
                }
            });
        }

        private static List<int> SearchProducts(List<FieldFilterModel> filters)
        {
            return pimFacade.Products.Search(new SearchModel
            {
                IncludeArchived = true,
                QueryModel = new SimpleQueryModel
                {
                    BooleanOperator = BooleanOperator.And,
                    Filters = filters
                }
            });
        }

    }
}
